@@ -1,23 +1,20 @@
 /*
- *
- * readconf.c
- *
  * Author: Tatu Ylonen <ylo@cs.hut.fi>
- *
  * Copyright (c) 1995 Tatu Ylonen <ylo@cs.hut.fi>, Espoo, Finland
  *                    All rights reserved
- *
- * Created: Sat Apr 22 00:03:10 1995 ylo
- *
  * Functions for reading the configuration files.
  *
+ * As far as I am concerned, the code I have written for this software
+ * can be used freely for any purpose.  Any derived versions of this
+ * software must be clearly marked as such, and if the derived work is
+ * incompatible with the protocol description in the RFC file, it must be
+ * called by a name other than "ssh" or "Secure Shell".
  */
 
 #include "includes.h"
-RCSID("$Id: readconf.c,v 1.31 2000/05/08 17:12:15 markus Exp $");
+RCSID("$OpenBSD: readconf.c,v 1.49 2000/10/11 20:27:23 markus Exp $");
 
 #include "ssh.h"
-#include "cipher.h"
 #include "readconf.h"
 #include "match.h"
 #include "xmalloc.h"
@@ -92,7 +89,7 @@ typedef enum {
 	oBadOption,
 	oForwardAgent, oForwardX11, oGatewayPorts, oRhostsAuthentication,
 	oPasswordAuthentication, oRSAAuthentication, oFallBackToRsh, oUseRsh,
-	oSkeyAuthentication,
+	oSkeyAuthentication, oXAuthLocation,
 #ifdef KRB4
 	oKerberosAuthentication,
 #endif /* KRB4 */
@@ -105,7 +102,8 @@ typedef enum {
 	oBatchMode, oCheckHostIP, oStrictHostKeyChecking, oCompression,
 	oCompressionLevel, oKeepAlives, oNumberOfPasswordPrompts, oTISAuthentication,
 	oUsePrivilegedPort, oLogLevel, oCiphers, oProtocol, oIdentityFile2,
-	oGlobalKnownHostsFile2, oUserKnownHostsFile2, oDSAAuthentication
+	oGlobalKnownHostsFile2, oUserKnownHostsFile2, oDSAAuthentication,
+	oKbdInteractiveAuthentication, oKbdInteractiveDevices
 } OpCodes;
 
 /* Textual representations of the tokens. */
@@ -116,10 +114,13 @@ static struct {
 } keywords[] = {
 	{ "forwardagent", oForwardAgent },
 	{ "forwardx11", oForwardX11 },
+	{ "xauthlocation", oXAuthLocation },
 	{ "gatewayports", oGatewayPorts },
 	{ "useprivilegedport", oUsePrivilegedPort },
 	{ "rhostsauthentication", oRhostsAuthentication },
 	{ "passwordauthentication", oPasswordAuthentication },
+	{ "kbdinteractiveauthentication", oKbdInteractiveAuthentication },
+	{ "kbdinteractivedevices", oKbdInteractiveDevices },
 	{ "rsaauthentication", oRSAAuthentication },
 	{ "dsaauthentication", oDSAAuthentication },
 	{ "skeyauthentication", oSkeyAuthentication },
@@ -163,10 +164,6 @@ static struct {
 	{ NULL, 0 }
 };
 
-/* Characters considered whitespace in strtok calls. */
-#define WHITESPACE " \t\r\n"
-
-
 /*
  * Adds a local TCP/IP port forward to options.  Never returns if there is an
  * error.
@@ -236,18 +233,20 @@ process_config_line(Options *options, const char *host,
 		    char *line, const char *filename, int linenum,
 		    int *activep)
 {
-	char buf[256], *cp, *string, **charptr, *cp2;
+	char buf[256], *s, *string, **charptr, *endofnumber, *keyword, *arg;
 	int opcode, *intptr, value;
 	u_short fwd_port, fwd_host_port;
 
-	/* Skip leading whitespace. */
-	cp = line + strspn(line, WHITESPACE);
-	if (!*cp || *cp == '\n' || *cp == '#')
+	s = line;
+	/* Get the keyword. (Each line is supposed to begin with a keyword). */
+	keyword = strdelim(&s);
+	/* Ignore leading whitespace. */
+	if (*keyword == '\0')
+		keyword = strdelim(&s);
+	if (!*keyword || *keyword == '\n' || *keyword == '#')
 		return 0;
 
-	/* Get the keyword. (Each line is supposed to begin with a keyword). */
-	cp = strtok(cp, WHITESPACE);
-	opcode = parse_token(cp, filename, linenum);
+	opcode = parse_token(keyword, filename, linenum);
 
 	switch (opcode) {
 	case oBadOption:
@@ -257,13 +256,13 @@ process_config_line(Options *options, const char *host,
 	case oForwardAgent:
 		intptr = &options->forward_agent;
 parse_flag:
-		cp = strtok(NULL, WHITESPACE);
-		if (!cp)
+		arg = strdelim(&s);
+		if (!arg || *arg == '\0')
 			fatal("%.200s line %d: Missing yes/no argument.", filename, linenum);
 		value = 0;	/* To avoid compiler warning... */
-		if (strcmp(cp, "yes") == 0 || strcmp(cp, "true") == 0)
+		if (strcmp(arg, "yes") == 0 || strcmp(arg, "true") == 0)
 			value = 1;
-		else if (strcmp(cp, "no") == 0 || strcmp(cp, "false") == 0)
+		else if (strcmp(arg, "no") == 0 || strcmp(arg, "false") == 0)
 			value = 0;
 		else
 			fatal("%.200s line %d: Bad yes/no argument.", filename, linenum);
@@ -291,6 +290,14 @@ parse_flag:
 		intptr = &options->password_authentication;
 		goto parse_flag;
 
+	case oKbdInteractiveAuthentication:
+		intptr = &options->kbd_interactive_authentication;
+		goto parse_flag;
+
+	case oKbdInteractiveDevices:
+		charptr = &options->kbd_interactive_devices;
+		goto parse_string;
+
 	case oDSAAuthentication:
 		intptr = &options->dsa_authentication;
 		goto parse_flag;
@@ -343,16 +350,16 @@ parse_flag:
 
 	case oStrictHostKeyChecking:
 		intptr = &options->strict_host_key_checking;
-		cp = strtok(NULL, WHITESPACE);
-		if (!cp)
+		arg = strdelim(&s);
+		if (!arg || *arg == '\0')
 			fatal("%.200s line %d: Missing yes/no argument.",
 			      filename, linenum);
 		value = 0;	/* To avoid compiler warning... */
-		if (strcmp(cp, "yes") == 0 || strcmp(cp, "true") == 0)
+		if (strcmp(arg, "yes") == 0 || strcmp(arg, "true") == 0)
 			value = 1;
-		else if (strcmp(cp, "no") == 0 || strcmp(cp, "false") == 0)
+		else if (strcmp(arg, "no") == 0 || strcmp(arg, "false") == 0)
 			value = 0;
-		else if (strcmp(cp, "ask") == 0)
+		else if (strcmp(arg, "ask") == 0)
 			value = 2;
 		else
 			fatal("%.200s line %d: Bad yes/no/ask argument.", filename, linenum);
@@ -378,8 +385,8 @@ parse_flag:
 
 	case oIdentityFile:
 	case oIdentityFile2:
-		cp = strtok(NULL, WHITESPACE);
-		if (!cp)
+		arg = strdelim(&s);
+		if (!arg || *arg == '\0')
 			fatal("%.200s line %d: Missing argument.", filename, linenum);
 		if (*activep) {
 			intptr = (opcode == oIdentityFile) ?
@@ -391,19 +398,23 @@ parse_flag:
 			charptr = (opcode == oIdentityFile) ?
 			    &options->identity_files[*intptr] :
 			    &options->identity_files2[*intptr];
-			*charptr = xstrdup(cp);
+			*charptr = xstrdup(arg);
 			*intptr = *intptr + 1;
 		}
 		break;
 
+	case oXAuthLocation:
+		charptr=&options->xauth_location;
+		goto parse_string;
+
 	case oUser:
 		charptr = &options->user;
 parse_string:
-		cp = strtok(NULL, WHITESPACE);
-		if (!cp)
+		arg = strdelim(&s);
+		if (!arg || *arg == '\0')
 			fatal("%.200s line %d: Missing argument.", filename, linenum);
 		if (*activep && *charptr == NULL)
-			*charptr = xstrdup(cp);
+			*charptr = xstrdup(arg);
 		break;
 
 	case oGlobalKnownHostsFile:
@@ -429,10 +440,10 @@ parse_string:
 	case oProxyCommand:
 		charptr = &options->proxy_command;
 		string = xstrdup("");
-		while ((cp = strtok(NULL, WHITESPACE)) != NULL) {
-			string = xrealloc(string, strlen(string) + strlen(cp) + 2);
+		while ((arg = strdelim(&s)) != NULL && *arg != '\0') {
+			string = xrealloc(string, strlen(string) + strlen(arg) + 2);
 			strcat(string, " ");
-			strcat(string, cp);
+			strcat(string, arg);
 		}
 		if (*activep && *charptr == NULL)
 			*charptr = string;
@@ -443,15 +454,15 @@ parse_string:
 	case oPort:
 		intptr = &options->port;
 parse_int:
-		cp = strtok(NULL, WHITESPACE);
-		if (!cp)
+		arg = strdelim(&s);
+		if (!arg || *arg == '\0')
 			fatal("%.200s line %d: Missing argument.", filename, linenum);
-		if (cp[0] < '0' || cp[0] > '9')
+		if (arg[0] < '0' || arg[0] > '9')
 			fatal("%.200s line %d: Bad number.", filename, linenum);
 
 		/* Octal, decimal, or hex format? */
-		value = strtol(cp, &cp2, 0);
-		if (cp == cp2)
+		value = strtol(arg, &endofnumber, 0);
+		if (arg == endofnumber)
 			fatal("%.200s line %d: Bad number.", filename, linenum);
 		if (*activep && *intptr == -1)
 			*intptr = value;
@@ -463,59 +474,65 @@ parse_int:
 
 	case oCipher:
 		intptr = &options->cipher;
-		cp = strtok(NULL, WHITESPACE);
-		value = cipher_number(cp);
+		arg = strdelim(&s);
+		if (!arg || *arg == '\0')
+			fatal("%.200s line %d: Missing argument.", filename, linenum);
+		value = cipher_number(arg);
 		if (value == -1)
 			fatal("%.200s line %d: Bad cipher '%s'.",
-			      filename, linenum, cp ? cp : "<NONE>");
+			      filename, linenum, arg ? arg : "<NONE>");
 		if (*activep && *intptr == -1)
 			*intptr = value;
 		break;
 
 	case oCiphers:
-		cp = strtok(NULL, WHITESPACE);
-		if (!ciphers_valid(cp))
+		arg = strdelim(&s);
+		if (!arg || *arg == '\0')
+			fatal("%.200s line %d: Missing argument.", filename, linenum);
+		if (!ciphers_valid(arg))
 			fatal("%.200s line %d: Bad SSH2 cipher spec '%s'.",
-			      filename, linenum, cp ? cp : "<NONE>");
+			      filename, linenum, arg ? arg : "<NONE>");
 		if (*activep && options->ciphers == NULL)
-			options->ciphers = xstrdup(cp);
+			options->ciphers = xstrdup(arg);
 		break;
 
 	case oProtocol:
 		intptr = &options->protocol;
-		cp = strtok(NULL, WHITESPACE);
-		value = proto_spec(cp);
+		arg = strdelim(&s);
+		if (!arg || *arg == '\0')
+			fatal("%.200s line %d: Missing argument.", filename, linenum);
+		value = proto_spec(arg);
 		if (value == SSH_PROTO_UNKNOWN)
 			fatal("%.200s line %d: Bad protocol spec '%s'.",
-			      filename, linenum, cp ? cp : "<NONE>");
+			      filename, linenum, arg ? arg : "<NONE>");
 		if (*activep && *intptr == SSH_PROTO_UNKNOWN)
 			*intptr = value;
 		break;
 
 	case oLogLevel:
 		intptr = (int *) &options->log_level;
-		cp = strtok(NULL, WHITESPACE);
-		value = log_level_number(cp);
+		arg = strdelim(&s);
+		value = log_level_number(arg);
 		if (value == (LogLevel) - 1)
 			fatal("%.200s line %d: unsupported log level '%s'\n",
-			      filename, linenum, cp ? cp : "<NONE>");
+			      filename, linenum, arg ? arg : "<NONE>");
 		if (*activep && (LogLevel) * intptr == -1)
 			*intptr = (LogLevel) value;
 		break;
 
 	case oRemoteForward:
-		cp = strtok(NULL, WHITESPACE);
-		if (!cp)
+		arg = strdelim(&s);
+		if (!arg || *arg == '\0')
 			fatal("%.200s line %d: Missing argument.", filename, linenum);
-		if (cp[0] < '0' || cp[0] > '9')
+		if (arg[0] < '0' || arg[0] > '9')
 			fatal("%.200s line %d: Badly formatted port number.",
 			      filename, linenum);
-		fwd_port = atoi(cp);
-		cp = strtok(NULL, WHITESPACE);
-		if (!cp)
+		fwd_port = atoi(arg);
+		arg = strdelim(&s);
+		if (!arg || *arg == '\0')
 			fatal("%.200s line %d: Missing second argument.",
 			      filename, linenum);
-		if (sscanf(cp, "%255[^:]:%hu", buf, &fwd_host_port) != 2)
+		if (sscanf(arg, "%255[^:]:%hu", buf, &fwd_host_port) != 2)
 			fatal("%.200s line %d: Badly formatted host:port.",
 			      filename, linenum);
 		if (*activep)
@@ -523,18 +540,18 @@ parse_int:
 		break;
 
 	case oLocalForward:
-		cp = strtok(NULL, WHITESPACE);
-		if (!cp)
+		arg = strdelim(&s);
+		if (!arg || *arg == '\0')
 			fatal("%.200s line %d: Missing argument.", filename, linenum);
-		if (cp[0] < '0' || cp[0] > '9')
+		if (arg[0] < '0' || arg[0] > '9')
 			fatal("%.200s line %d: Badly formatted port number.",
 			      filename, linenum);
-		fwd_port = atoi(cp);
-		cp = strtok(NULL, WHITESPACE);
-		if (!cp)
+		fwd_port = atoi(arg);
+		arg = strdelim(&s);
+		if (!arg || *arg == '\0')
 			fatal("%.200s line %d: Missing second argument.",
 			      filename, linenum);
-		if (sscanf(cp, "%255[^:]:%hu", buf, &fwd_host_port) != 2)
+		if (sscanf(arg, "%255[^:]:%hu", buf, &fwd_host_port) != 2)
 			fatal("%.200s line %d: Badly formatted host:port.",
 			      filename, linenum);
 		if (*activep)
@@ -543,26 +560,26 @@ parse_int:
 
 	case oHost:
 		*activep = 0;
-		while ((cp = strtok(NULL, WHITESPACE)) != NULL)
-			if (match_pattern(host, cp)) {
-				debug("Applying options for %.100s", cp);
+		while ((arg = strdelim(&s)) != NULL && *arg != '\0')
+			if (match_pattern(host, arg)) {
+				debug("Applying options for %.100s", arg);
 				*activep = 1;
 				break;
 			}
-		/* Avoid garbage check below, as strtok already returned NULL. */
+		/* Avoid garbage check below, as strdelim is done. */
 		return 0;
 
 	case oEscapeChar:
 		intptr = &options->escape_char;
-		cp = strtok(NULL, WHITESPACE);
-		if (!cp)
+		arg = strdelim(&s);
+		if (!arg || *arg == '\0')
 			fatal("%.200s line %d: Missing argument.", filename, linenum);
-		if (cp[0] == '^' && cp[2] == 0 &&
-		    (unsigned char) cp[1] >= 64 && (unsigned char) cp[1] < 128)
-			value = (unsigned char) cp[1] & 31;
-		else if (strlen(cp) == 1)
-			value = (unsigned char) cp[0];
-		else if (strcmp(cp, "none") == 0)
+		if (arg[0] == '^' && arg[2] == 0 &&
+		    (unsigned char) arg[1] >= 64 && (unsigned char) arg[1] < 128)
+			value = (unsigned char) arg[1] & 31;
+		else if (strlen(arg) == 1)
+			value = (unsigned char) arg[0];
+		else if (strcmp(arg, "none") == 0)
 			value = -2;
 		else {
 			fatal("%.200s line %d: Bad escape character.",
@@ -579,9 +596,11 @@ parse_int:
 	}
 
 	/* Check that there is no garbage at end of line. */
-	if (strtok(NULL, WHITESPACE) != NULL)
-		fatal("%.200s line %d: garbage at end of line.",
-		      filename, linenum);
+	if ((arg = strdelim(&s)) != NULL && *arg != '\0')
+	{
+		fatal("%.200s line %d: garbage at end of line; \"%.200s\".",
+		      filename, linenum, arg);
+	}
 	return 0;
 }
 
@@ -638,6 +657,7 @@ initialize_options(Options * options)
 	memset(options, 'X', sizeof(*options));
 	options->forward_agent = -1;
 	options->forward_x11 = -1;
+	options->xauth_location = NULL;
 	options->gateway_ports = -1;
 	options->use_privileged_port = -1;
 	options->rhosts_authentication = -1;
@@ -652,6 +672,8 @@ initialize_options(Options * options)
 	options->afs_token_passing = -1;
 #endif
 	options->password_authentication = -1;
+	options->kbd_interactive_authentication = -1;
+	options->kbd_interactive_devices = NULL;
 	options->rhosts_rsa_authentication = -1;
 	options->fallback_to_rsh = -1;
 	options->use_rsh = -1;
@@ -691,9 +713,13 @@ void
 fill_default_options(Options * options)
 {
 	if (options->forward_agent == -1)
-		options->forward_agent = 1;
+		options->forward_agent = 0;
 	if (options->forward_x11 == -1)
 		options->forward_x11 = 0;
+#ifdef XAUTH_PATH
+	if (options->xauth_location == NULL)
+		options->xauth_location = XAUTH_PATH;
+#endif /* XAUTH_PATH */
 	if (options->gateway_ports == -1)
 		options->gateway_ports = 0;
 	if (options->use_privileged_port == -1)
@@ -718,10 +744,12 @@ fill_default_options(Options * options)
 #endif /* AFS */
 	if (options->password_authentication == -1)
 		options->password_authentication = 1;
+	if (options->kbd_interactive_authentication == -1)
+		options->kbd_interactive_authentication = 0;
 	if (options->rhosts_rsa_authentication == -1)
 		options->rhosts_rsa_authentication = 1;
 	if (options->fallback_to_rsh == -1)
-		options->fallback_to_rsh = 1;
+		options->fallback_to_rsh = 0;
 	if (options->use_rsh == -1)
 		options->use_rsh = 0;
 	if (options->batch_mode == -1)
